/****************************************************************************
 * (c) 2019 softwareentwicklung-als-prozess.de
 * Funktionalitaet:
 * Frequenz messen und ausgeben mit einem ATMega328
 ***************************************************************************/
// Includes -----------------------------------------------------------------
#include "i2clcd.h"
#include "input_capture.h"

// Defines -------------------------------------------------------------------
#define TXTLEN 17
// nur einen von beiden!!
//#define TERMINAL  // Auswerten mit Tabellenkalkulation
#define PLOT      // Auswerten mit seriellem Plotter

// Variablen -----------------------------------------------------------------
char     txt[TXTLEN];         // Ausgabetext
uint16_t   f = (uint16_t)0;   // Frequenz in Hz
uint8_t frac =  (uint8_t)0;   // Nachkommastellen der Frequenz
uint32_t   T = (uint32_t)0;   // Periodendauer in us
double  freq =   (double)0;   // Frequenz inklusive Nachkommastellen
uint16_t   t = (uint16_t)0;   // Messzeit in s

// interne Prototypen --------------------------------------------------------
static void cleartext(void);  // Textfeld loeschen

// Funktionen ----------------------------------------------------------------
void setup() 
{
  Serial.begin(115200);                   // serielle Ausgabe auf Terminalprogramm
#ifdef TERMINAL
  Serial.println(F("Fuer Tabellenkalkulation"));
#endif
#ifdef PLOT
  Serial.println(F("Fuer seriellen Plotter"));
#endif
  Serial.println(F("Frequenzmessung"));
  Serial.println(F("Zeit[s] f[Hz] "));
  i2c_lcd_Setup();                        // Ausgabe auf LCD
  sprintf(txt,"Frequenzmessung:");
  i2c_lcd_Write(I2CLCD_LINE1,txt);
  cleartext();
  sprintf(txt,"f = %5u.%1u Hz  ",f,frac);
  i2c_lcd_Write(I2CLCD_LINE2,txt);
  cleartext();
  inpcap_Init();                          // Frequenzzaehler
}

void loop() 
{
  // Messwerte holen
  if ((bool)false == inpcap_Found())
  {
    T = inpcap_GetPeriodendauer();
    freq = 1.0 / ((double)T / (double)1000000.0);
  }
  else
  {
    T    = (uint32_t)0;
    freq = (double)0;
  }
  
  // Ausgabe auf LCD und Terminal
  Serial.print(t, DEC);
  Serial.print(" ");
  f    = (uint16_t)freq;                                                      // Vor komma
  frac = (uint16_t)((freq * (double)10.0) - ((uint16_t)freq * (uint16_t)10)); // eine Stelle nach Komma
  if (((double)31.0 > freq) || ((double)99.0 < freq))
  {
    sprintf(txt, "ausser Bereich! ");
    Serial.println("0 ");
    i2c_lcd_Write(I2CLCD_LINE2,txt);
    cleartext();
  }
  else // Ausgabe nur im Bereich von 31 bis 99 Hz
  {
    sprintf(txt, "f = %5u.%1u Hz  ",f, frac);  
    i2c_lcd_Write(I2CLCD_LINE2,txt);
    cleartext();
#ifdef PLOT
    Serial.println(freq,DEC); // Ausgabe mit Punkt fuer Plot
#endif
#ifdef TERMINAL
    sprintf(txt, "%5u,%1u ",f, frac);  
    Serial.print(txt);       // Ausgabe mit Komma fuer Calc
    Serial.println(F(" "));
    cleartext();
#endif
    f    = (uint16_t)0;
    frac = (uint16_t)0;
  }
  t++; // naechste sekunde
  delay(1000);
}

// Textausgabefeld initialisieren
static void cleartext(void)
{
  int i = 0;

  for(i = 0; i < (TXTLEN - 1); i++)
  {
    txt[i] = EMPTY;
  }
  txt[TXTLEN - 1] = ENDE;
}
