/****************************************************************************
 * (c) 2019 softwareentwicklung-als-prozess.de
 * Funktionalitaet:
 * Frequenzmessung per Input Capture auf dem Timer1 mit einem ATMega328
 ***************************************************************************/
// Includes -----------------------------------------------------------------
#include "Arduino.h"
#include "input_capture.h"

// Defines -------------------------------------------------------------------

// Variablen -----------------------------------------------------------------
volatile static uint8_t  ovfcnt       =  (uint8_t)0; // Ueberlaufzaehler
volatile static uint8_t  flankedetekt =  (uint8_t)0; // Flanke erkannt
volatile static uint16_t cntval0      = (uint16_t)0; // alter zaehlerwert
volatile static uint16_t cntval1      = (uint16_t)0; // neuer zaehlerwert
volatile static uint32_t Cntval       = (uint32_t)0; // periodendauer in zaehler
volatile static bool     notfound     =  (bool)false;

// Funktionen ----------------------------------------------------------------
void inpcap_Init(void)
{
  // mindestfrequenz fuer ATMega328p
  // Takt             8 MHz     16 MHz
  // Prescale /8      15,26     30,52 Hz 
  // Prescale /64     1,9       3,8 Hz
  // Prescale /256    0,48      0,96 Hz
  // Prescale /1024   0,12      0,24 Hz
  // hoechstfrequenz fuer ATMega328p
  // Takt             8 MHz     16 MHz
  // Prescale /8      1e06      2e06 Hz 
  // Prescale /64     125e03    250e03 Hz
  // Prescale /256    31,25e03  62,5e03 Hz
  // Prescale /1024   7812      15625 Hz
  TCCR1A = 0;
  TCCR1B = (0 << ICNC1) | (0 << ICES1) | (1 << CS11); // Noise Canceler an, Trigger auf fallende Flanke, Prescaler /8
  TCCR1C = 0;
  TCCR1B &= ~(1 << ICES1);                            // fallende Flanke
  TIFR1  = (1 << ICF1)  | (1 << TOV1);                // anstehende Interrupts loeschen
  TIMSK1 = (1 << ICIE1) | (1 << TOIE1);               // interrupts aktivieren
  pinMode(8, INPUT);                                  // Input Capture Pin auf Pin D8
  digitalWrite(8, 1);                                 // mit Pullup
}

uint32_t inpcap_GetPeriodendauer(void)
{
  // Zeit je Zaehler
  // Takt             8 MHz     16 MHz
  // Prescale /8      1         0,5 us
  // Prescale /64     8         4 us
  // Prescale /256    32        16 us
  // Prescale /1024   128       64 us
  return(Cntval / 2); // 16 MHz (Uno), Prescaler /8
}

bool inpcap_Found(void)
{
  return(notfound);
}

ISR(TIMER1_OVF_vect)                /* Ueberlauf Interrupt */
{
  ovfcnt++;
  if ((uint8_t)10 < ovfcnt)         /* mehrfacher ueberlauf */
  {                                 /* keine weitere Flanke aufgetreten */
    cntval0  = (uint16_t)0;
    cntval1  = (uint16_t)0;
    Cntval   = (uint32_t)0;
    ovfcnt   =  (uint8_t)0;
    notfound =  (bool)true;
    flankedetekt = (uint8_t)0;
  }
}

ISR(TIMER1_CAPT_vect)               /* Input Capture Interrupt */
{
  if (flankedetekt == 0)            /* Test auf erste flanke */  
  {    
    cntval0 = (ICR1H << 8) + ICR1L; /* einlesen */
    flankedetekt = 1;               /* erste flanke gesehen */  
    notfound = (bool)false;         /* einen Wert eingelesen */
  }  
  else  
  {
    cntval1 = (ICR1H << 8) + ICR1L; /* einlesen */
    if (cntval1 > cntval0)          /* auf ueberlauf pruefen */    
    {
      Cntval = cntval1 - cntval0;   /* kein Ueberlauf */ 
    }    
    else
    {                               /* ein Ueberlauf */
      Cntval = cntval1 + 0xFFFF - cntval0 + 1;
      ovfcnt = (uint8_t)0;          /* ueberlauf zaehler reset */
    }
    cntval0 = cntval1;              /* naechste periode */
  } 
}
